@@ -31,6 +31,7 @@ sgn_NewContext(SECOidTag alg, SECItem *params, SECKEYPrivateKey *key)
     SGNContext *cx;
     SECOidTag hashalg, signalg;
     KeyType keyType;
+    PRUint32 policyFlags;
     SECStatus rv;
 
     /* OK, map a PKCS #7 hash and encrypt algorithm into
@@ -44,7 +45,7 @@ sgn_NewContext(SECOidTag alg, SECItem *params, SECKEYPrivateKey *key)
     rv = sec_DecodeSigAlg(NULL, alg, params, &signalg, &hashalg);
     if (rv != SECSuccess) {
         PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
-        return 0;
+        return NULL;
     }
     keyType = seckey_GetKeyType(signalg);
 
@@ -53,7 +54,19 @@ sgn_NewContext(SECOidTag alg, SECItem *params, SECKEYPrivateKey *key)
         !((key->keyType == dsaKey) && (keyType == fortezzaKey)) &&
         !((key->keyType == rsaKey) && (keyType == rsaPssKey))) {
         PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
-        return 0;
+        return NULL;
+    }
+    /* check the policy on the hash algorithm */
+    if ((NSS_GetAlgorithmPolicy(hashalg, &policyFlags) == SECFailure) ||
+        !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
+        PORT_SetError(SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
+        return NULL;
+    }
+    /* check the policy on the encryption algorithm */
+    if ((NSS_GetAlgorithmPolicy(signalg, &policyFlags) == SECFailure) ||
+        !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
+        PORT_SetError(SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
+        return NULL;
     }
 
     cx = (SGNContext *)PORT_ZAlloc(sizeof(SGNContext));
@@ -452,9 +465,27 @@ SGN_Digest(SECKEYPrivateKey *privKey,
     SECItem digder;
     PLArenaPool *arena = 0;
     SGNDigestInfo *di = 0;
+    SECOidTag enctag;
+    PRUint32 policyFlags;
 
     result->data = 0;
 
+    /* check the policy on the hash algorithm */
+    if ((NSS_GetAlgorithmPolicy(algtag, &policyFlags) == SECFailure) ||
+        !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
+        PORT_SetError(SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
+        return SECFailure;
+    }
+    /* check the policy on the encryption algorithm */
+    enctag = sec_GetEncAlgFromSigAlg(
+        SEC_GetSignatureAlgorithmOidTag(privKey->keyType, algtag));
+    if ((enctag == SEC_OID_UNKNOWN) ||
+        (NSS_GetAlgorithmPolicy(enctag, &policyFlags) == SECFailure) ||
+        !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
+        PORT_SetError(SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
+        return SECFailure;
+    }
+
     if (privKey->keyType == rsaKey) {
 
         arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
