@@ -174,30 +174,35 @@ Hacl_Poly1305_32_poly1305_update1(uint64_t *ctx, uint8_t *text)
     uint64_t t2 = a26;
     uint64_t t3 = a36;
     uint64_t t4 = a46;
-    uint64_t l = t0 + (uint64_t)0U;
-    uint64_t tmp0 = l & (uint64_t)0x3ffffffU;
-    uint64_t c0 = l >> (uint32_t)26U;
-    uint64_t l0 = t1 + c0;
-    uint64_t tmp1 = l0 & (uint64_t)0x3ffffffU;
-    uint64_t c1 = l0 >> (uint32_t)26U;
-    uint64_t l1 = t2 + c1;
-    uint64_t tmp2 = l1 & (uint64_t)0x3ffffffU;
-    uint64_t c2 = l1 >> (uint32_t)26U;
-    uint64_t l2 = t3 + c2;
-    uint64_t tmp3 = l2 & (uint64_t)0x3ffffffU;
-    uint64_t c3 = l2 >> (uint32_t)26U;
-    uint64_t l3 = t4 + c3;
-    uint64_t tmp4 = l3 & (uint64_t)0x3ffffffU;
-    uint64_t c4 = l3 >> (uint32_t)26U;
-    uint64_t l4 = tmp0 + c4 * (uint64_t)5U;
-    uint64_t tmp01 = l4 & (uint64_t)0x3ffffffU;
-    uint64_t c5 = l4 >> (uint32_t)26U;
-    uint64_t tmp11 = tmp1 + c5;
-    uint64_t o0 = tmp01;
-    uint64_t o1 = tmp11;
-    uint64_t o2 = tmp2;
-    uint64_t o3 = tmp3;
-    uint64_t o4 = tmp4;
+    uint64_t mask26 = (uint64_t)0x3ffffffU;
+    uint64_t z0 = t0 >> (uint32_t)26U;
+    uint64_t z1 = t3 >> (uint32_t)26U;
+    uint64_t x0 = t0 & mask26;
+    uint64_t x3 = t3 & mask26;
+    uint64_t x1 = t1 + z0;
+    uint64_t x4 = t4 + z1;
+    uint64_t z01 = x1 >> (uint32_t)26U;
+    uint64_t z11 = x4 >> (uint32_t)26U;
+    uint64_t t = z11 << (uint32_t)2U;
+    uint64_t z12 = z11 + t;
+    uint64_t x11 = x1 & mask26;
+    uint64_t x41 = x4 & mask26;
+    uint64_t x2 = t2 + z01;
+    uint64_t x01 = x0 + z12;
+    uint64_t z02 = x2 >> (uint32_t)26U;
+    uint64_t z13 = x01 >> (uint32_t)26U;
+    uint64_t x21 = x2 & mask26;
+    uint64_t x02 = x01 & mask26;
+    uint64_t x31 = x3 + z02;
+    uint64_t x12 = x11 + z13;
+    uint64_t z03 = x31 >> (uint32_t)26U;
+    uint64_t x32 = x31 & mask26;
+    uint64_t x42 = x41 + z03;
+    uint64_t o0 = x02;
+    uint64_t o1 = x12;
+    uint64_t o2 = x21;
+    uint64_t o3 = x32;
+    uint64_t o4 = x42;
     acc[0U] = o0;
     acc[1U] = o1;
     acc[2U] = o2;
@@ -211,8 +216,8 @@ Hacl_Poly1305_32_poly1305_update(uint64_t *ctx, uint32_t len, uint8_t *text)
     uint64_t *pre = ctx + (uint32_t)5U;
     uint64_t *acc = ctx;
     uint32_t nb = len / (uint32_t)16U;
-    uint32_t rem1 = len % (uint32_t)16U;
-    for (uint32_t i = (uint32_t)0U; i < nb; i = i + (uint32_t)1U) {
+    uint32_t rem = len % (uint32_t)16U;
+    for (uint32_t i = (uint32_t)0U; i < nb; i++) {
         uint8_t *block = text + i * (uint32_t)16U;
         uint64_t e[5U] = { 0U };
         uint64_t u0 = load64_le(block);
@@ -296,41 +301,46 @@ Hacl_Poly1305_32_poly1305_update(uint64_t *ctx, uint32_t len, uint8_t *text)
         uint64_t t2 = a26;
         uint64_t t3 = a36;
         uint64_t t4 = a46;
-        uint64_t l = t0 + (uint64_t)0U;
-        uint64_t tmp0 = l & (uint64_t)0x3ffffffU;
-        uint64_t c0 = l >> (uint32_t)26U;
-        uint64_t l0 = t1 + c0;
-        uint64_t tmp1 = l0 & (uint64_t)0x3ffffffU;
-        uint64_t c1 = l0 >> (uint32_t)26U;
-        uint64_t l1 = t2 + c1;
-        uint64_t tmp2 = l1 & (uint64_t)0x3ffffffU;
-        uint64_t c2 = l1 >> (uint32_t)26U;
-        uint64_t l2 = t3 + c2;
-        uint64_t tmp3 = l2 & (uint64_t)0x3ffffffU;
-        uint64_t c3 = l2 >> (uint32_t)26U;
-        uint64_t l3 = t4 + c3;
-        uint64_t tmp4 = l3 & (uint64_t)0x3ffffffU;
-        uint64_t c4 = l3 >> (uint32_t)26U;
-        uint64_t l4 = tmp0 + c4 * (uint64_t)5U;
-        uint64_t tmp01 = l4 & (uint64_t)0x3ffffffU;
-        uint64_t c5 = l4 >> (uint32_t)26U;
-        uint64_t tmp11 = tmp1 + c5;
-        uint64_t o0 = tmp01;
-        uint64_t o1 = tmp11;
-        uint64_t o2 = tmp2;
-        uint64_t o3 = tmp3;
-        uint64_t o4 = tmp4;
+        uint64_t mask26 = (uint64_t)0x3ffffffU;
+        uint64_t z0 = t0 >> (uint32_t)26U;
+        uint64_t z1 = t3 >> (uint32_t)26U;
+        uint64_t x0 = t0 & mask26;
+        uint64_t x3 = t3 & mask26;
+        uint64_t x1 = t1 + z0;
+        uint64_t x4 = t4 + z1;
+        uint64_t z01 = x1 >> (uint32_t)26U;
+        uint64_t z11 = x4 >> (uint32_t)26U;
+        uint64_t t = z11 << (uint32_t)2U;
+        uint64_t z12 = z11 + t;
+        uint64_t x11 = x1 & mask26;
+        uint64_t x41 = x4 & mask26;
+        uint64_t x2 = t2 + z01;
+        uint64_t x01 = x0 + z12;
+        uint64_t z02 = x2 >> (uint32_t)26U;
+        uint64_t z13 = x01 >> (uint32_t)26U;
+        uint64_t x21 = x2 & mask26;
+        uint64_t x02 = x01 & mask26;
+        uint64_t x31 = x3 + z02;
+        uint64_t x12 = x11 + z13;
+        uint64_t z03 = x31 >> (uint32_t)26U;
+        uint64_t x32 = x31 & mask26;
+        uint64_t x42 = x41 + z03;
+        uint64_t o0 = x02;
+        uint64_t o1 = x12;
+        uint64_t o2 = x21;
+        uint64_t o3 = x32;
+        uint64_t o4 = x42;
         acc[0U] = o0;
         acc[1U] = o1;
         acc[2U] = o2;
         acc[3U] = o3;
         acc[4U] = o4;
     }
-    if (rem1 > (uint32_t)0U) {
-        uint8_t *last1 = text + nb * (uint32_t)16U;
+    if (rem > (uint32_t)0U) {
+        uint8_t *last = text + nb * (uint32_t)16U;
         uint64_t e[5U] = { 0U };
         uint8_t tmp[16U] = { 0U };
-        memcpy(tmp, last1, rem1 * sizeof last1[0U]);
+        memcpy(tmp, last, rem * sizeof(last[0U]));
         uint64_t u0 = load64_le(tmp);
         uint64_t lo = u0;
         uint64_t u = load64_le(tmp + (uint32_t)8U);
@@ -352,10 +362,10 @@ Hacl_Poly1305_32_poly1305_update(uint64_t *ctx, uint32_t len, uint8_t *text)
         e[2U] = f2;
         e[3U] = f3;
         e[4U] = f4;
-        uint64_t b = (uint64_t)1U << rem1 * (uint32_t)8U % (uint32_t)26U;
+        uint64_t b = (uint64_t)1U << rem * (uint32_t)8U % (uint32_t)26U;
         uint64_t mask = b;
-        uint64_t fi = e[rem1 * (uint32_t)8U / (uint32_t)26U];
-        e[rem1 * (uint32_t)8U / (uint32_t)26U] = fi | mask;
+        uint64_t fi = e[rem * (uint32_t)8U / (uint32_t)26U];
+        e[rem * (uint32_t)8U / (uint32_t)26U] = fi | mask;
         uint64_t *r = pre;
         uint64_t *r5 = pre + (uint32_t)5U;
         uint64_t r0 = r[0U];
@@ -412,30 +422,35 @@ Hacl_Poly1305_32_poly1305_update(uint64_t *ctx, uint32_t len, uint8_t *text)
         uint64_t t2 = a26;
         uint64_t t3 = a36;
         uint64_t t4 = a46;
-        uint64_t l = t0 + (uint64_t)0U;
-        uint64_t tmp0 = l & (uint64_t)0x3ffffffU;
-        uint64_t c0 = l >> (uint32_t)26U;
-        uint64_t l0 = t1 + c0;
-        uint64_t tmp1 = l0 & (uint64_t)0x3ffffffU;
-        uint64_t c1 = l0 >> (uint32_t)26U;
-        uint64_t l1 = t2 + c1;
-        uint64_t tmp2 = l1 & (uint64_t)0x3ffffffU;
-        uint64_t c2 = l1 >> (uint32_t)26U;
-        uint64_t l2 = t3 + c2;
-        uint64_t tmp3 = l2 & (uint64_t)0x3ffffffU;
-        uint64_t c3 = l2 >> (uint32_t)26U;
-        uint64_t l3 = t4 + c3;
-        uint64_t tmp4 = l3 & (uint64_t)0x3ffffffU;
-        uint64_t c4 = l3 >> (uint32_t)26U;
-        uint64_t l4 = tmp0 + c4 * (uint64_t)5U;
-        uint64_t tmp01 = l4 & (uint64_t)0x3ffffffU;
-        uint64_t c5 = l4 >> (uint32_t)26U;
-        uint64_t tmp11 = tmp1 + c5;
-        uint64_t o0 = tmp01;
-        uint64_t o1 = tmp11;
-        uint64_t o2 = tmp2;
-        uint64_t o3 = tmp3;
-        uint64_t o4 = tmp4;
+        uint64_t mask26 = (uint64_t)0x3ffffffU;
+        uint64_t z0 = t0 >> (uint32_t)26U;
+        uint64_t z1 = t3 >> (uint32_t)26U;
+        uint64_t x0 = t0 & mask26;
+        uint64_t x3 = t3 & mask26;
+        uint64_t x1 = t1 + z0;
+        uint64_t x4 = t4 + z1;
+        uint64_t z01 = x1 >> (uint32_t)26U;
+        uint64_t z11 = x4 >> (uint32_t)26U;
+        uint64_t t = z11 << (uint32_t)2U;
+        uint64_t z12 = z11 + t;
+        uint64_t x11 = x1 & mask26;
+        uint64_t x41 = x4 & mask26;
+        uint64_t x2 = t2 + z01;
+        uint64_t x01 = x0 + z12;
+        uint64_t z02 = x2 >> (uint32_t)26U;
+        uint64_t z13 = x01 >> (uint32_t)26U;
+        uint64_t x21 = x2 & mask26;
+        uint64_t x02 = x01 & mask26;
+        uint64_t x31 = x3 + z02;
+        uint64_t x12 = x11 + z13;
+        uint64_t z03 = x31 >> (uint32_t)26U;
+        uint64_t x32 = x31 & mask26;
+        uint64_t x42 = x41 + z03;
+        uint64_t o0 = x02;
+        uint64_t o1 = x12;
+        uint64_t o2 = x21;
+        uint64_t o3 = x32;
+        uint64_t o4 = x42;
         acc[0U] = o0;
         acc[1U] = o1;
         acc[2U] = o2;
@@ -451,47 +466,64 @@ Hacl_Poly1305_32_poly1305_finish(uint8_t *tag, uint8_t *key, uint64_t *ctx)
     uint64_t *acc = ctx;
     uint8_t *ks = key + (uint32_t)16U;
     uint64_t f0 = acc[0U];
-    uint64_t f12 = acc[1U];
-    uint64_t f22 = acc[2U];
-    uint64_t f32 = acc[3U];
+    uint64_t f13 = acc[1U];
+    uint64_t f23 = acc[2U];
+    uint64_t f33 = acc[3U];
     uint64_t f40 = acc[4U];
-    uint64_t l = f0 + (uint64_t)0U;
+    uint64_t l0 = f0 + (uint64_t)0U;
+    uint64_t tmp00 = l0 & (uint64_t)0x3ffffffU;
+    uint64_t c00 = l0 >> (uint32_t)26U;
+    uint64_t l1 = f13 + c00;
+    uint64_t tmp10 = l1 & (uint64_t)0x3ffffffU;
+    uint64_t c10 = l1 >> (uint32_t)26U;
+    uint64_t l2 = f23 + c10;
+    uint64_t tmp20 = l2 & (uint64_t)0x3ffffffU;
+    uint64_t c20 = l2 >> (uint32_t)26U;
+    uint64_t l3 = f33 + c20;
+    uint64_t tmp30 = l3 & (uint64_t)0x3ffffffU;
+    uint64_t c30 = l3 >> (uint32_t)26U;
+    uint64_t l4 = f40 + c30;
+    uint64_t tmp40 = l4 & (uint64_t)0x3ffffffU;
+    uint64_t c40 = l4 >> (uint32_t)26U;
+    uint64_t f010 = tmp00 + c40 * (uint64_t)5U;
+    uint64_t f110 = tmp10;
+    uint64_t f210 = tmp20;
+    uint64_t f310 = tmp30;
+    uint64_t f410 = tmp40;
+    uint64_t l = f010 + (uint64_t)0U;
     uint64_t tmp0 = l & (uint64_t)0x3ffffffU;
     uint64_t c0 = l >> (uint32_t)26U;
-    uint64_t l0 = f12 + c0;
-    uint64_t tmp1 = l0 & (uint64_t)0x3ffffffU;
-    uint64_t c1 = l0 >> (uint32_t)26U;
-    uint64_t l1 = f22 + c1;
-    uint64_t tmp2 = l1 & (uint64_t)0x3ffffffU;
-    uint64_t c2 = l1 >> (uint32_t)26U;
-    uint64_t l2 = f32 + c2;
-    uint64_t tmp3 = l2 & (uint64_t)0x3ffffffU;
-    uint64_t c3 = l2 >> (uint32_t)26U;
-    uint64_t l3 = f40 + c3;
-    uint64_t tmp4 = l3 & (uint64_t)0x3ffffffU;
-    uint64_t c4 = l3 >> (uint32_t)26U;
-    uint64_t l4 = tmp0 + c4 * (uint64_t)5U;
-    uint64_t tmp0_ = l4 & (uint64_t)0x3ffffffU;
-    uint64_t c5 = l4 >> (uint32_t)26U;
-    uint64_t f010 = tmp0_;
-    uint64_t f110 = tmp1 + c5;
-    uint64_t f210 = tmp2;
-    uint64_t f310 = tmp3;
-    uint64_t f410 = tmp4;
+    uint64_t l5 = f110 + c0;
+    uint64_t tmp1 = l5 & (uint64_t)0x3ffffffU;
+    uint64_t c1 = l5 >> (uint32_t)26U;
+    uint64_t l6 = f210 + c1;
+    uint64_t tmp2 = l6 & (uint64_t)0x3ffffffU;
+    uint64_t c2 = l6 >> (uint32_t)26U;
+    uint64_t l7 = f310 + c2;
+    uint64_t tmp3 = l7 & (uint64_t)0x3ffffffU;
+    uint64_t c3 = l7 >> (uint32_t)26U;
+    uint64_t l8 = f410 + c3;
+    uint64_t tmp4 = l8 & (uint64_t)0x3ffffffU;
+    uint64_t c4 = l8 >> (uint32_t)26U;
+    uint64_t f02 = tmp0 + c4 * (uint64_t)5U;
+    uint64_t f12 = tmp1;
+    uint64_t f22 = tmp2;
+    uint64_t f32 = tmp3;
+    uint64_t f42 = tmp4;
     uint64_t mh = (uint64_t)0x3ffffffU;
     uint64_t ml = (uint64_t)0x3fffffbU;
-    uint64_t mask = FStar_UInt64_eq_mask(f410, mh);
-    uint64_t mask1 = mask & FStar_UInt64_eq_mask(f310, mh);
-    uint64_t mask2 = mask1 & FStar_UInt64_eq_mask(f210, mh);
-    uint64_t mask3 = mask2 & FStar_UInt64_eq_mask(f110, mh);
-    uint64_t mask4 = mask3 & ~~FStar_UInt64_gte_mask(f010, ml);
+    uint64_t mask = FStar_UInt64_eq_mask(f42, mh);
+    uint64_t mask1 = mask & FStar_UInt64_eq_mask(f32, mh);
+    uint64_t mask2 = mask1 & FStar_UInt64_eq_mask(f22, mh);
+    uint64_t mask3 = mask2 & FStar_UInt64_eq_mask(f12, mh);
+    uint64_t mask4 = mask3 & ~~FStar_UInt64_gte_mask(f02, ml);
     uint64_t ph = mask4 & mh;
     uint64_t pl = mask4 & ml;
-    uint64_t o0 = f010 - pl;
-    uint64_t o1 = f110 - ph;
-    uint64_t o2 = f210 - ph;
-    uint64_t o3 = f310 - ph;
-    uint64_t o4 = f410 - ph;
+    uint64_t o0 = f02 - pl;
+    uint64_t o1 = f12 - ph;
+    uint64_t o2 = f22 - ph;
+    uint64_t o3 = f32 - ph;
+    uint64_t o4 = f42 - ph;
     uint64_t f011 = o0;
     uint64_t f111 = o1;
     uint64_t f211 = o2;
