@@ -92,6 +92,11 @@ sftkdb_passwordToKey(SFTKDBHandle *keydb, SECItem *salt,
     SHA1Context *cx = NULL;
     SECStatus rv = SECFailure;
 
+    if (!pw) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+        return SECFailure;
+    }
+
     key->data = PORT_Alloc(SHA1_LENGTH);
     if (key->data == NULL) {
         goto loser;
@@ -1384,7 +1389,7 @@ sftkdb_ChangePassword(SFTKDBHandle *keydb,
     certdb = keydb->peerDB;
     if (certdb) {
         CK_ATTRIBUTE objectType = { CKA_CLASS, 0, sizeof(CK_OBJECT_CLASS) };
-        CK_OBJECT_CLASS myClass = CKO_NETSCAPE_TRUST;
+        CK_OBJECT_CLASS myClass = CKO_NSS_TRUST;
 
         objectType.pValue = &myClass;
         crv = sftkdb_convertObjects(certdb, &objectType, 1, &newKey,
